#ifndef CONSTRAINT_HH
#define CONSTRAINT_HH

enum constraint_types
{
  move_dist,
  stretch_dist,
  facing_change,
  facing_away,
  body_x_motion,
  body_y_motion,
  body_motion,
  body_x_pos,
  body_y_pos,
  num_constraint_types
};

enum variation_types
{
  no_variation,
  wave_sin,
  wave_cos,
  wave_tan,
  num_variation_types
};

char const* const constraint_type_names[num_constraint_types]=
{
  "move_dist",
  "stretch_dist",
  "facing_change",
  "facing_away",
  "body_x_motion",
  "body_y_motion",
  "body_motion",
  "body_x_pos",
  "body_y_pos",
  "goalness",
};

struct variation_data_t
{
  int type;
  double offset;
  double phase;
  double amplitude;
  double frequency;
  // frequency = pi / 180 is one step per degree
  double eval(double step)
  {
    switch(type)
    {
      case no_variation:
        return offset;
      case wave_sin:
        return offset + (amplitude * sin(phase + (frequency * step)));
      case wave_cos:
        return offset + (amplitude * cos(phase + (frequency * step)));
      case wave_tan:
        return offset + (amplitude * tan(phase + (frequency * step)));
    }
    return offset;
  }
};

struct constraint_t
{
  unsigned int type;
  // TODO:  Switch to using a comparison function the way the STL does.
  bool is_max_constraint;  // A constraint can be a max or a min.
  bool has_edge_case;
  double current_value;
  double edge_sensitivity;  // A case that is betweeen edge_sensitivity and
  // value is considered an edge condition.
  double goal_value;
  double change_rate;
  double after_edge_value;

  bool allow_step(double stat);
  void check_edge_and_react(double stat);
  void recover_from_edge();
};

struct weight_changer_t
{
  unsigned int type; // Same type values as constraint types.
  bool is_gt_compare;
  double compare_value;
  int weight_change_amount;
  variation_data_t weight_change_variation;

  int weight_step(double stat, size_t step_count);
};

#endif
