#include <iostream>
using std::cout;
using std::endl;
#include <cmath>

size_t const num_itg_panels= 8;

/*
char const* const itg_panel_names[num_itg_panels+1]=
{
  "a_left",
  "a_down",
  "a_up",
  "a_right",
  "b_left",
  "b_down",
  "b_up",
  "b_right",
  "num_itg_panels"
};
*/

double const itg_panel_positions[num_itg_panels*2]=
{
  -2.5, 1.5,-1.5, 0.5,-1.5, 2.5,-0.5, 1.5,
  0.5, 1.5,1.5, 0.5,1.5, 2.5,2.5, 2.5
};

size_t const num_pump_panels= 10;

/*
char const* const pump_panel_names[num_pump_panels+1]=
{
  "a_ddwn_left",
  "a_up_left",
  "a_center",
  "a_up_right",
  "a_down_right",
  "b_ddwn_left",
  "b_up_left",
  "b_center",
  "b_up_right",
  "b_down_right",
  "num_pump_panels"
};
*/

double const pump_panel_positions[num_pump_panels*2]=
{
  -2.5, .5,-2.5, 2.5,-1.5, 1.5,-0.5, 2.5,-0.5, 0.5,
  0.5, 0.5,0.5, 2.5,1.5, 1.5,2.5, 0.5,2.5, 2.5
};

typedef void (*calc_function_t)(double ax, double ay, double bx, double by);

void output_two_d_constant_set(calc_function_t func,
  double const* const positions,
  size_t const num_positions, char const* const set_name)
  {
    cout << "double const " << set_name << "[num_panels][num_panels]=" << endl
      << "{" << endl;
    for(size_t r= 0; r < num_positions; ++r)
    {
      cout << "{";
      double ax= positions[r*2];
      double ay= positions[(r*2)+1];
      for(size_t c= 0; c < num_positions; ++c)
      {
        double bx= positions[c*2];
        double by= positions[(c*2)+1];
        func(ax, ay, bx, by);
        if(c < num_positions - 1)
        {
          cout << ", ";
        }
      }
      cout << "}";
      if(r < num_positions - 1)
      {
        cout << ",";
      }
      cout << endl;
    }
    cout << "};" << endl;
  }


void calc_separation(double ax, double ay, double bx, double by)
  {
    double xd= ax - bx;
    double yd= ay - by;
    double separation= sqrt((xd * xd) + (yd * yd));
    cout << separation;
  }

void sub(double ax, double ay, double bx, double by, double& rx, double& ry)
  {
    rx= bx - ax;
    ry= by - ay;
  }

void calc_facing_direction(double ax, double ay, double bx, double by)
  {
    double rightx;
    double righty;
    sub(ax, ay, bx, by, rightx, righty);
    double right_angle= atan2(righty, rightx);
    double forward_angle= right_angle + M_PI;
    if(forward_angle > (2 * M_PI))
    {
      forward_angle-= 2 * M_PI;
    }
    cout << forward_angle;
  }

void calc_body_position(double ax, double ay, double bx, double by)
  {
    double avgx= (ax + bx) / 2.0;
    double avgy= (ay + by) / 2.0;
    cout << "{" << avgx << ", " << avgy << "}";
  }

void output_style_data(double const* const positions,
  size_t const num_positions, char const* const style_name)
  {
    cout << "namespace " << style_name << endl
      << "{" << endl
      << "size_t const num_panels= " << num_positions << ";" << endl
      << endl;
    output_two_d_constant_set(&calc_separation, positions,
      num_positions, "panel_positions");
    cout << endl;
    output_two_d_constant_set(&calc_facing_direction, positions,
      num_positions, "facing_directions");
    cout << endl;
    output_two_d_constant_set(&calc_body_position, positions,
      num_positions, "body_positions");
    cout << endl
      << "};" << endl;
  }

int main()
  {
    output_style_data(itg_panel_positions, num_itg_panels, "itg");
    cout << endl;
    output_style_data(pump_panel_positions, num_pump_panels, "pump");
    return 0;
  }
